@@ -2,6 +2,8 @@
 #ifdef LINUX2
 #define HAVE_DAEMON
 #define HAVE_GETRUSAGE
+#define HAVE_SNPRINTF
+#define HAVE_VSNPRINTF
 #define SOCKLEN_T	socklen_t
 #define PATH_TMP	"/tmp"
 #define PATH_VAR_TMP	"/var/tmp"
@@ -14,6 +16,8 @@
 
 #ifdef HPUX10
 #define HAVE_GETRUSAGE
+#define HAVE_SNPRINTF
+#define HAVE_VSNPRINTF
 #define SOCKLEN_T	int
 #define PATH_TMP	"/tmp"
 #define PATH_VAR_TMP	"/var/tmp"
@@ -26,6 +30,8 @@
 
 #ifdef TRU64
 #define HAVE_GETRUSAGE
+#define HAVE_SNPRINTF
+#define HAVE_VSNPRINTF
 #define SIZEOF_IS_LONG_INT
 #define SOCKLEN_T	size_t
 #define PATH_TMP	"/tmp"
@@ -61,7 +67,7 @@
 /* not supported */
 #define SA_RESTART 0
 #define O_RSYNC 0
-#ifdef NEXTSTEP3_HPPA
+#if defined(NEXTSTEP3_HPPA) || defined(NEXTSTEP4)
 /* replace with corresponding bsd call */
 #define setsid() setpgrp(0, getpid())
 #else /* NEXTSTEP3_HPPA */
@@ -109,6 +115,8 @@
 
 #ifdef UNIXWARE7
 #define HAVE_GETRUSAGE
+#define HAVE_SNPRINTF
+#define HAVE_VSNPRINTF
 #define PID_T_IS_LONG_INT
 #define SOCKLEN_T	size_t
 #define O_RSYNC		0
@@ -130,6 +138,8 @@
 
 #ifdef SCO
 #define HAVE_GETRUSAGE
+#define HAVE_SNPRINTF
+#define HAVE_VSNPRINTF
 #define SOCKLEN_T	int
 #define O_RSYNC		0
 #define PATH_TMP        "/tmp"
