@@ -19,7 +19,7 @@
 
 #include "prngd.h"
 
-int randsavefd = -1;
+static int randsavefd = -1;
 int debug = 0;
 
 int devnull = -1;
@@ -33,11 +33,15 @@ static void open_seedfile(const char *seedfile)
 {
   int bytes, bytes_read_total;
   unsigned char buffer[PRNGD_STATE_SIZE];
+  char msg_buf[1024];
 
   randsavefd = open(seedfile, O_RDWR | O_CREAT | O_SYNC | O_RSYNC, 400);
   if (randsavefd < 0)
   {
-    fprintf(stderr, "Could not open %s: %s\n", seedfile, strerror(errno));
+    (void)snprintf(msg_buf, sizeof(msg_buf),
+	"could not open %s: %s\n", seedfile, strerror(errno));
+    msg_buf[sizeof(msg_buf) - 1] = '\0';
+    error_fatal(msg_buf, 0);
     exit(EXIT_FAILURE);
   }
   close_on_exec(randsavefd, 1);
@@ -48,7 +52,10 @@ static void open_seedfile(const char *seedfile)
    */
   if (fchmod(randsavefd, S_IRUSR | S_IWUSR) < 0)
   {
-    fprintf(stderr, "Could not chmod %s: %s\n", seedfile, strerror(errno));
+    (void)snprintf(msg_buf, sizeof(msg_buf),
+	"Could not chmod %s: %s\n", seedfile, strerror(errno));
+    msg_buf[sizeof(msg_buf) - 1] = '\0';
+    error_fatal(msg_buf, 0);
     exit(EXIT_FAILURE);
   }
 
@@ -68,7 +75,10 @@ static void open_seedfile(const char *seedfile)
   }
   if (bytes < 0)
   {
-    fprintf(stderr, "Error reading %s: %s\n", seedfile, strerror(errno));
+    (void)snprintf(msg_buf, sizeof(msg_buf),
+	"Error reading %s: %s\n", seedfile, strerror(errno));
+    msg_buf[sizeof(msg_buf) - 1] = '\0';
+    error_fatal(msg_buf, 0);
     exit(EXIT_FAILURE);
   }
 
@@ -78,7 +88,7 @@ static void open_seedfile(const char *seedfile)
   seed_internal(NULL);
 
   if (debug)
-    fprintf(stdout, "Read %d bytes\n", bytes_read_total);
+    (void)fprintf(stderr, "Read %d bytes\n", bytes_read_total);
   /*
    * Finally, account for the bytes read. We don't know how much entropy
    * was in the bytes we've read, so we only account for the minimum needed
@@ -106,25 +116,25 @@ static void write_seedfile(void)
   bytes = lseek(randsavefd, 0, SEEK_SET);
   if (bytes < 0)
   {
-    fprintf(stderr, "Error writing seedfile: %s\n", strerror(errno));
+    error_fatal("Error writing seedfile", errno);
     exit(EXIT_FAILURE);
   }
   bytes = rand_bytes(buffer, PRNGD_STATE_SIZE);
   if (bytes == 0)
   {
-    fprintf(stderr, "Info: Random pool not (yet) seeded\n");
+    error_error("Info: Random pool not (yet) seeded", 0);
   }
   else
   {
     bytes = write(randsavefd, buffer, PRNGD_STATE_SIZE);
     if (bytes < 0)
     {
-      fprintf(stderr, "Error writing seedfile: %s\n", strerror(errno));
+      error_fatal("Error writing seedfile", errno);
       exit(EXIT_FAILURE);
     }
   }
   if (debug)
-    fprintf(stdout, "Wrote %d bytes back to seedfile\n", bytes);
+    (void)fprintf(stderr, "Wrote %d bytes back to seedfile\n", bytes);
   bytes = ftruncate(randsavefd, bytes);
 }
 
@@ -132,7 +142,7 @@ void close_seedfile(void)
 {
   if (randsavefd >= 0) {
     write_seedfile();
-    close(randsavefd);
+    (void)close(randsavefd);
   }
 }
 
@@ -140,41 +150,46 @@ void close_seedfile(void)
 static void killpeer(int service_socket)
 {
   unsigned char buffer[256];
+  char msg_buf[1024];
   int num;
   pid_t peer_id;
 
   buffer[0] = 0x04;
   if (write(service_socket, buffer, 1) < 1)
   {
-    fprintf(stderr, "Cannot contact peer: %s\n", strerror(errno));
+    error_fatal("Cannot contact peer", errno);
     exit(EXIT_FAILURE);
   }
   if (read(service_socket, buffer, 1) < 1)
   {
-    fprintf(stderr, "Cannot read from peer: %s\n", strerror(errno));
+    error_fatal("Cannot read from peer", errno);
     exit(EXIT_FAILURE);
   }
   num = buffer[0];
   if (read(service_socket, buffer, num) < num)
   {
-    fprintf(stderr, "Cannot read from peer: %s\n", strerror(errno));
+    error_fatal("Cannot read from peer", errno);
     exit(EXIT_FAILURE);
   }
   buffer[num] = '\0';
 #ifdef PID_T_IS_LONG_INT
   peer_id = atol((char *) buffer);
-  fprintf(stderr, "Killing: %ld\n", peer_id);
+  (void)fprintf(stderr, "Killing: %ld\n", peer_id);
 #else
   peer_id = atoi((char *) buffer);
-  fprintf(stderr, "Killing: %d\n", peer_id);
+  (void)fprintf(stderr, "Killing: %d\n", (int)peer_id);
 #endif
 
   if (kill(peer_id, SIGTERM) < 0)
   {
 #ifdef PID_T_IS_LONG_INT
-    fprintf(stderr, "Cannot kill %ld: %s\n", peer_id, strerror(errno));
+    (void)snprintf(msg_buf, sizeof(msg_buf), "Cannot kill %ld", peer_id);
+    msg_buf[sizeof(msg_buf) - 1] = '\0';
+    error_fatal(msg_buf, errno);
 #else
-    fprintf(stderr, "Cannot kill %d: %s\n", peer_id, strerror(errno));
+    (void)snprintf(msg_buf, sizeof(msg_buf), "Cannot kill %d", (int)peer_id);
+    msg_buf[sizeof(msg_buf) - 1] = '\0';
+    error_fatal(msg_buf, errno);
 #endif
     exit(EXIT_FAILURE);
   }
@@ -183,8 +198,8 @@ static void killpeer(int service_socket)
 
 static void usage(const char *progname)
 {
-  fprintf(stderr, "prngd %s (%s)\n", PRNGD_VERSION, PRNGD_DATE);
-  fprintf(stderr,
+  (void)fprintf(stderr, "prngd %s (%s)\n", PRNGD_VERSION, PRNGD_DATE);
+  (void)fprintf(stderr,
 	  "Usage: %s [options] /path/to/socket1 [/path/to/socket2 ...]\nOptions:\n\t-d/--debug: debugging on\n\t-c/--cmdfile cmdpath: use cmdpath for entropy commands [%s]\n\t-s/--seedfile seedpath: use seedpath as seedfile [%s]\n\t-n/--no-seedfile: no seedfile, keep pool in memory only\n\t-m/--mode mode: use mode for sockets [%04o]\n\t-k/--kill: kill daemon on other side\n\t-v/--version: print version and exit\n",
 	  progname, CONFIGFILE, RANDSAVENAME, SOCKETMODE);
 }
@@ -201,6 +216,7 @@ int main(int argc, char *argv[])
   char **randsock = NULL;
   char *cmdpath = CONFIGFILE;
   char *seedpath = RANDSAVENAME;
+  char msg_buf[1024];
   unsigned int socketmode = SOCKETMODE;
   int *service_socket, kill_socket;
   struct sockaddr_un sockun, sockun_kill;
@@ -216,7 +232,7 @@ int main(int argc, char *argv[])
   while (argp < argc)
   {
     if (!strcmp(argv[argp], "-v") || !strcmp(argv[argp], "--version")) {
-      fprintf(stdout, "prngd %s (%s)\n", PRNGD_VERSION, PRNGD_DATE);
+      (void)fprintf(stdout, "prngd %s (%s)\n", PRNGD_VERSION, PRNGD_DATE);
       exit(EXIT_SUCCESS);
     }
     else if (!strcmp(argv[argp], "-d") || !strcmp(argv[argp], "--debug"))
@@ -232,7 +248,10 @@ int main(int argc, char *argv[])
     {
       if (sscanf(argv[++argp], "%o", &socketmode) == 0)
       {
-	fprintf(stderr, "Cannot interprete socketmode '%s'\n", argv[argp]);
+	(void)snprintf(msg_buf, sizeof(msg_buf),
+		"Cannot interpret socket mode '%s'\n", argv[argp]);
+	msg_buf[sizeof(msg_buf) - 1] = '\0';
+	error_fatal(msg_buf, 0);
 	exit(EXIT_FAILURE);
       }
     }
@@ -246,7 +265,7 @@ int main(int argc, char *argv[])
       else
 	randsock = realloc(randsock, (numsock + 1) * sizeof(char *));
       if (randsock == NULL) {
-	fprintf(stderr, "Could not allocate memory: %s\n", strerror(errno));
+	error_fatal("Could not allocate memory", errno);
 	exit(EXIT_FAILURE);
       }
       randsock[numsock++] = argv[argp];
@@ -268,11 +287,11 @@ int main(int argc, char *argv[])
 
   if (!numsock)
   {
-    fprintf(stderr, "No socket name given, exiting!\n");
+    error_fatal("no socket name given, exiting!", 0);
     exit(EXIT_FAILURE);
   }
   if (debug)
-    fprintf(stdout, "Debugging enabled\n");
+    (void)fprintf(stdout, "Debugging enabled\n");
 
   /*
    * Obtain the maximum number of open filedescriptors and adjust the
@@ -287,7 +306,7 @@ int main(int argc, char *argv[])
    */
   devnull = open("/dev/null", O_RDWR);
   if (devnull == -1) {
-    fprintf(stderr, "Couldn't open /dev/null: %s\n", strerror(errno));
+    error_fatal("Couldn't open /dev/null", errno);
     exit(EXIT_FAILURE);
   }
   count_openfd++;
@@ -295,10 +314,8 @@ int main(int argc, char *argv[])
   /*
    * Now, initialize the PRNG (allocate the memory).
    */
-  if (!rand_init(PRNGD_STATE_SIZE, ENTROPY_NEEDED)) {
-    fprintf(stderr, "Could not initialize PRNG\n");
-    exit(EXIT_FAILURE);
-  }
+  rand_init(PRNGD_STATE_SIZE, ENTROPY_NEEDED);
+
 
   /*
    * Start by reading back the seed saved from former runs, if usage of a
@@ -310,7 +327,7 @@ int main(int argc, char *argv[])
   }
 
   if (!(service_socket = malloc(numsock * sizeof(int)))) {
-    fprintf(stderr, "Could not allocate memory: %s\n", strerror(errno));
+    error_fatal("Could not allocate memory", errno);
     exit(EXIT_FAILURE);
   }
 
@@ -320,7 +337,7 @@ int main(int argc, char *argv[])
    */
   kill_socket = socket(AF_UNIX, SOCK_STREAM, 0);
   if (kill_socket < 0) {
-    fprintf(stderr, "Could not create socket: %s\n", strerror(errno));
+    error_fatal( "Could not create socket", errno);
     exit(EXIT_FAILURE);
   }
   count_openfd++;
@@ -337,15 +354,20 @@ int main(int argc, char *argv[])
      * socket! If removal fails for whatever reason, stop, as something
      * may be seriously wrong.
      */
-    if ((statval = stat(randsock[i], &randsock_stat) < 0) &&
-	(errno != ENOENT)) {
-      fprintf(stderr, "Cannot stat socket position %s: %s\n", randsock[i],
-	      strerror(errno));
+    statval = stat(randsock[i], &randsock_stat);
+    if ((statval < 0) && (errno != ENOENT)) {
+      (void)snprintf(msg_buf, sizeof(msg_buf), "Cannot stat socket position %s",
+      	randsock[i]);
+      msg_buf[sizeof(msg_buf) - 1] = '\0';
+      error_fatal(msg_buf, errno);
       exit(EXIT_FAILURE);
     }
     else if (!statval) {
       if (!S_ISSOCK(randsock_stat.st_mode)) {
-	fprintf(stderr, "Will not touch %s: no socket\n", randsock[i]);
+	(void)snprintf(msg_buf, sizeof(msg_buf), "Will not touch %s: no socket",
+		randsock[i]);
+	msg_buf[sizeof(msg_buf) - 1] = '\0';
+	error_fatal(msg_buf, errno);
 	exit(EXIT_FAILURE);
       } else {
 	/*
@@ -354,24 +376,33 @@ int main(int argc, char *argv[])
 	 * and we are going to kill it. When not in kill mode, we cannot
 	 * start a new daemon at this address and exit with error.
 	 */
-	memset(&sockun, 0, sizeof(sockun_kill));
+	(void)memset(&sockun, 0, sizeof(sockun_kill));
 	sockun_kill.sun_family = AF_UNIX;
 	if (sizeof(sockun_kill.sun_path) < strlen(randsock[i]) + 1) {
 #ifdef SIZEOF_IS_LONG_INT
-	  fprintf(stderr, "Sockfilename too long: %s, maxlen=%ld\n",
-		  randsock[i], sizeof(sockun_kill.sun_path));
+	(void)snprintf(msg_buf, sizeof(msg_buf),
+		"Sockfilename too long: %s, maxlen=%ld\n", randsock[i],
+		sizeof(sockun_kill.sun_path));
+	msg_buf[sizeof(msg_buf) - 1] = '\0';
+	error_error(msg_buf);
 #else
-	  fprintf(stderr, "Sockfilename too long: %s, maxlen=%d\n",
-		  randsock[i], sizeof(sockun_kill.sun_path));
+	(void)snprintf(msg_buf, sizeof(msg_buf),
+		"Sockfilename too long: %s, maxlen=%d\n", randsock[i],
+		sizeof(sockun_kill.sun_path));
+	msg_buf[sizeof(msg_buf) - 1] = '\0';
+	error_error(msg_buf, 0);
 #endif /* SIZEOF_IS_LONG_INT */
 	  exit(EXIT_FAILURE);
         }
 
-	strcpy(sockun_kill.sun_path, randsock[i]);
+	(void)strcpy(sockun_kill.sun_path, randsock[i]);
 	if (connect(kill_socket, (struct sockaddr *) &sockun_kill,
 		    sizeof(sockun_kill)) < 0) {
 	  if (killmode) {
-	    fprintf(stderr, "Cannot kill peer at %s: no daemon\n", randsock[i]);
+	    (void)snprintf(msg_buf, sizeof(msg_buf),
+	    	"cannot kill peer at %s: no daemon", randsock[i]);
+	    msg_buf[sizeof(msg_buf) - 1] = '\0';
+	    error_error(msg_buf, 0);
 	    /*
 	     * Do no exit because there may be more work to do!
 	     */
@@ -380,17 +411,19 @@ int main(int argc, char *argv[])
 	}
 	else if (killmode) {
 	  killpeer(kill_socket);
-	  shutdown(kill_socket, SHUT_RDWR);
+	  my_shutdown(kill_socket, SHUT_RDWR);
 	  count_openfd--;
 	  continue;
 	}
 	else {
-	  fprintf(stderr,
-		  "Socket %s already used by another process, exiting.\n",
+	  (void)snprintf(msg_buf, sizeof(msg_buf),
+		  "socket %s already used by another process, exiting.\n",
 		  randsock[i]);
+	  msg_buf[sizeof(msg_buf) - 1] = '\0';
+	  error_fatal(msg_buf, 0);
 	  exit(EXIT_FAILURE);
 	}
-	shutdown(kill_socket, SHUT_RDWR);
+	my_shutdown(kill_socket, SHUT_RDWR);
 	count_openfd--;
 
 	/*
@@ -398,8 +431,10 @@ int main(int argc, char *argv[])
 	 * remove it.
 	 */
 	if (remove(randsock[i]) < 0) {
-	  fprintf(stderr, "Cannot remove already existing entry %s: %s\n",
-		  randsock[i], strerror(errno));
+	  (void)snprintf(msg_buf, sizeof(msg_buf),
+	  	"Cannot remove already existing entry %s", randsock[i]);
+	  msg_buf[sizeof(msg_buf) - 1] = '\0';
+	  error_fatal(msg_buf, errno);
 	  exit(EXIT_FAILURE);
 	}
       }
@@ -414,31 +449,35 @@ int main(int argc, char *argv[])
     service_socket[i] = socket(AF_UNIX, SOCK_STREAM, 0);
     if (service_socket[i] < 0)
     {
-      fprintf(stderr, "Could not create socket: %s\n", strerror(errno));
+      error_fatal("Could not create socket", errno);
       exit(EXIT_FAILURE);
     }
     /*
      * Bind the socket to the actual socket-file
      */
-    memset(&sockun, 0, sizeof(sockun));
+    (void)memset(&sockun, 0, sizeof(sockun));
     sockun.sun_family = AF_UNIX;
-    strcpy(sockun.sun_path, randsock[i]);
+    (void)strcpy(sockun.sun_path, randsock[i]);
     if (bind(service_socket[i], (struct sockaddr *) &sockun,
 	     sizeof(sockun)) < 0)
     {
-      fprintf(stderr, "Could not bind socket to %s: %s\n", randsock[i],
-	      strerror(errno));
+      (void)snprintf(msg_buf, sizeof(msg_buf), "Could not bind socket to %s",
+      	randsock[i]);
+      msg_buf[sizeof(msg_buf) - 1] = '\0';
+      error_fatal(msg_buf, errno);
       exit(EXIT_FAILURE);
     }
     if (chmod(randsock[i], socketmode) < 0)
     {
-      fprintf(stderr, "Could not chmod socket to %04o: %s\n", socketmode,
-	      strerror(errno));
+      (void)snprintf(msg_buf, sizeof(msg_buf), "Could not chmod socket to %04o",
+      	socketmode);
+      msg_buf[sizeof(msg_buf) - 1] = '\0';
+      error_fatal(msg_buf, errno);
       exit(EXIT_FAILURE);
     }
     if (listen(service_socket[i], SOMAXCONN) < 0)
     {
-      fprintf(stderr, "Could not listen on socket: %s\n", strerror(errno));
+      error_fatal("Could not listen on socket", errno);
       exit(EXIT_FAILURE);
     }
     /*
@@ -467,7 +506,7 @@ int main(int argc, char *argv[])
     ret = daemon(0, 0);
     if (ret < 0)
     {
-      fprintf(stderr, "Could not daemonize: %s\n", strerror(errno));
+      error_fatal("Could not daemonize", errno);
       exit(EXIT_FAILURE);
     }
 
@@ -477,18 +516,18 @@ int main(int argc, char *argv[])
      * The setup is finished, and we are daemonized, so we must now switch
      * to syslog...
      */
-    openlog("prngd", LOG_PID, LOG_DAEMON);
-    syslog(LOG_INFO, "prngd %s (%s) started up for user %s", PRNGD_VERSION,
-	   PRNGD_DATE, username);
+    (void)snprintf(msg_buf, sizeof(msg_buf),
+    	"prngd %s (%s) started up for user %s", PRNGD_VERSION, PRNGD_DATE,
+	username);
+    msg_buf[sizeof(msg_buf) - 1] = '\0';
+    error_notice(msg_buf, 0);
     count_openfd++;
-    syslog(LOG_INFO, "Have %d out of %d filedescriptors open\n", count_openfd,
-	    limit_openfd);
-
+    (void)snprintf(msg_buf, sizeof(msg_buf),
+    	"have %d out of %d filedescriptors open", count_openfd, limit_openfd);
+    msg_buf[sizeof(msg_buf) - 1] = '\0';
+    error_notice(msg_buf, 0);
   }
 
-  if (debug)
-    fprintf(stderr, "Have %d out of %d filedescriptors open\n", count_openfd,
-	    limit_openfd);
   /*
    * Maybe we have changed our pid, let's add some more of it. 
    */
