@@ -16,10 +16,10 @@
 
 #include "prngd.h"
 
-static seed_t gatherer_seed;
-static pid_t gatherer_pid = -1;
-static int gatherer_status;
-static int caught_sigchild = 0;
+static volatile seed_t gatherer_seed;
+static volatile pid_t sigchild_pid = -1;
+static volatile int gatherer_status;
+static volatile int caught_sigchild = 0;
 static volatile int received_sighup = 0;
 
 typedef enum
@@ -41,35 +41,43 @@ typedef struct
 } egd_query_t;
 
 
-static void termination_handler(int received_signal)
+static void termination_handler(int received_signal /*ARGSUSED*/)
 {
   if (debug)
-    fprintf(stderr, "Caught SIGTERM, shutting down\n");
+    (void)fprintf(stderr, "Caught SIGTERM, shutting down\n");
   close_seedfile();
   exit(EXIT_SUCCESS);
 }
 
-static void sighup_handler(int received_signal)
+static void sighup_handler(int received_signal /*ARGSUSED*/)
 {
   received_sighup = 1;
 }
 
-static void child_handler(int received_signal)
+static void child_handler(int received_signal /*ARGSUSED*/)
 {
   int save_errno = errno;
+  int my_gatherer_status;
   pid_t pid;
 
   /*
    * Wait for child process to avoid zombies. This signal comes in
-   * asynchronously and only few functions are save to be called,
+   * asynchronously and only few functions are safe to be called,
    * especially stdio and syslog functions cannot be called.
+   *
+   * PRNGD can only have one child at a time, so recording the information
+   * is trivial.
    */
   caught_sigchild = 1;
 
-  if ((pid = waitpid(-1, &gatherer_status, WNOHANG)) == -1)
-    ;  /* Hmm, how to log this error without syslog... */
-  else if ((pid == gatherer_pid) && WIFEXITED(gatherer_status))
-     gatherer_pid = -1;
+  do {
+    pid = waitpid(-1, &my_gatherer_status, WNOHANG);
+  } while (pid == -1 && errno == EINTR);
+
+  if (WIFEXITED(my_gatherer_status)) {
+    sigchild_pid = pid;
+    gatherer_status = my_gatherer_status;
+  }
 
   errno = save_errno;
 }
@@ -89,6 +97,7 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
   egd_query_t *egd_query = NULL;
   egd_query_t *egd_query_old;
   int max_fd;
+  pid_t gatherer_pid;
   int gather_fd;
   int gatherer_entry;
   int gatherer_bytes;
@@ -96,10 +105,12 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
   double est = 0.0;
   time_t gather_start;
   unsigned char gatherbuf[10240];
+  char msg_buf[1024];
   struct sigaction new_action;
   fd_set rfds, wfds, efds;
   struct timeval timeout;
 
+  gatherer_pid = -1;
   gather_fd = -1;
   gatherer_entry = 0;
 
@@ -114,29 +125,29 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
    * we don't want prngd to be affected in any way (it should simply close
    * the connection), so SIGPIPE shall be ignored.
    */
-  memset(&new_action, 0, sizeof(struct sigaction));
+  (void)memset(&new_action, 0, sizeof(struct sigaction));
   new_action.sa_handler = termination_handler;
-  sigemptyset(&new_action.sa_mask);
+  (void)sigemptyset(&new_action.sa_mask);
   new_action.sa_flags |= SA_RESTART;
-  sigaction(SIGTERM, &new_action, NULL);
+  (void)sigaction(SIGTERM, &new_action, NULL);
 
-  memset(&new_action, 0, sizeof(struct sigaction));
+  (void)memset(&new_action, 0, sizeof(struct sigaction));
   new_action.sa_handler = sighup_handler;
-  sigemptyset(&new_action.sa_mask);
+  (void)sigemptyset(&new_action.sa_mask);
   new_action.sa_flags |= SA_RESTART;
-  sigaction(SIGHUP, &new_action, NULL);
+  (void)sigaction(SIGHUP, &new_action, NULL);
 
-  memset(&new_action, 0, sizeof(struct sigaction));
+  (void)memset(&new_action, 0, sizeof(struct sigaction));
   new_action.sa_handler = child_handler;
-  sigemptyset(&new_action.sa_mask);
+  (void)sigemptyset(&new_action.sa_mask);
   new_action.sa_flags |= SA_RESTART;
-  sigaction(SIGCHLD, &new_action, NULL);
+  (void)sigaction(SIGCHLD, &new_action, NULL);
 
-  memset(&new_action, 0, sizeof(struct sigaction));
+  (void)memset(&new_action, 0, sizeof(struct sigaction));
   new_action.sa_handler = SIG_IGN;
-  sigemptyset(&new_action.sa_mask);
+  (void)sigemptyset(&new_action.sa_mask);
   new_action.sa_flags |= SA_RESTART;
-  sigaction(SIGPIPE, &new_action, NULL);
+  (void)sigaction(SIGPIPE, &new_action, NULL);
 
   for (;;)
   {
@@ -193,16 +204,24 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
     }
     num = select(max_fd + 1, &rfds, &wfds, &efds, &timeout);
 
+    /*
+     * Depending on the scheduling the SIGCHILD may already be old. This
+     * does not matter however, since we only have one child at a time
+     * and do not start another one before the old one has been killed
+     * (or at least tried to with SIGKILL).
+     */
     if (caught_sigchild) {
       caught_sigchild = 0;
       if (debug)
-	fprintf(stderr, "Caught SIGCHLD\n");
+	(void)fprintf(stderr, "Caught SIGCHLD\n");
+      if (sigchild_pid == gatherer_pid)
+	gatherer_pid = -1;
     }
 
     if (received_sighup) {
       received_sighup = 0;
       if (debug)
-	fprintf(stderr, "Received SIGHUP\n");
+	(void)fprintf(stderr, "Received SIGHUP\n");
     }
 
     if (time(0) >= last_seed_stat + seed_stat_interval) {
@@ -216,8 +235,8 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
     if (gather_fd != -1) {
       if (FD_ISSET(gather_fd, &efds)) {
 	if (debug)
-	  fprintf(stderr, "Closing connection to gatherer (efds)\n");
-	close(gather_fd);
+	  (void)fprintf(stderr, "Closing connection to gatherer (efds)\n");
+	(void)close(gather_fd);
 	gather_fd = -1;
 	count_openfd--;
       }
@@ -228,15 +247,15 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 	 */
 	numbytes = socket_read(gather_fd, gatherbuf, 10240);
 	rand_add(gatherbuf, numbytes, est * numbytes);
-	if (debug && numbytes > 0)
-	  fprintf(stderr, "Read %d bytes from gatherer\n", numbytes);
+	if (debug)
+	  (void)fprintf(stderr, "Read %d bytes from gatherer\n", numbytes);
 	gatherer_bytes += numbytes;
 	if ((numbytes <= 0) || (gatherer_bytes >= max_gatherer_bytes)) {
 	  if (debug)
-	    fprintf(stderr, "Closing connection to gatherer\n");
-	  close(gather_fd);
-	  rand_add(&gatherer_status, sizeof(gatherer_status), 0.0);
-	  rand_add(&gatherer_seed, sizeof(seed_t), 0.25);
+	    (void)fprintf(stderr, "Closing connection to gatherer\n");
+	  (void)close(gather_fd);
+	  rand_add((int *)&gatherer_status, sizeof(gatherer_status), 0.0);
+	  rand_add((seed_t *)&gatherer_seed, sizeof(seed_t), 0.25);
 	  gather_fd = -1;
 	  count_openfd--;
 	}
@@ -255,14 +274,27 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
     }
 
     /*
-     * If a gatherer has not finished in time, kill it!!
+     * If a gatherer has not finished in time, kill it!! Re-check, whether
+     * the signal came in in between. This might happen, since we close
+     * the connection when too many bytes where sent. If SIGCHILD is not
+     * there and the time is over, we kill anyway. If the time is not over,
+     * the check will be done at the next possible occation.
      */
+    if (caught_sigchild) {
+      caught_sigchild = 0;
+      if (debug)
+	(void)fprintf(stderr, "Caught SIGCHLD\n");
+      if (sigchild_pid == gatherer_pid)
+	gatherer_pid = -1;
+    }
     if ((gatherer_pid > 0) &&
 	(difftime(time(NULL), gather_start) > seed_ext_interval / 2 + 1)) {
-      kill(gatherer_pid, SIGKILL);
+      if (kill(gatherer_pid, SIGKILL) == -1) {
+      	error_error("kill() failed", errno);
+      }
       gatherer_pid = -1;
       if (gather_fd != -1) {
-	close(gather_fd);
+	(void)close(gather_fd);
 	gather_fd = -1;
 	count_openfd--;
 	gatherer_entry++;
@@ -279,13 +311,13 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
      */
     if (((time(0) >= last_seed_ext + seed_ext_interval) || first_gather ||
 	(rand_pool() < (ENTROPY_THRESHOLD))) &&
-	(gather_fd == -1))
+	(gatherer_pid == -1) && (gather_fd == -1))
     {
       last_seed_ext = time(0);
       /*
        * time to call another gatherer 
        */
-      if (pipe(pipedes) < 0)
+      if (my_pipe(pipedes) < 0)
 	continue;		/*
 				 * We cannot do anything about it 
 				 */
@@ -304,49 +336,62 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 
       gatherer_pid = fork();
       if (gatherer_pid == -1) {
-	if (debug)
-	  fprintf(stderr, "Could not fork: %s\n", strerror(errno));
-	close(pipedes[0]);
-	close(pipedes[1]);
+	error_error("Could not fork", errno);
+	(void)close(pipedes[0]);
+	(void)close(pipedes[1]);
 	continue;
       }
       if (gatherer_pid == 0) {
 	/*
          * Child
 	 */
-	dup2(devnull, STDIN_FILENO);
-        dup2(pipedes[1], STDOUT_FILENO);
-        dup2(pipedes[1], STDERR_FILENO);
-        close(pipedes[0]);
-        close(pipedes[1]);
-        close(devnull); 
-	execl(entropy_source[gatherer_entry].path,
+	my_dup2(devnull, STDIN_FILENO);
+        my_dup2(pipedes[1], STDOUT_FILENO);
+        my_dup2(pipedes[1], STDERR_FILENO);
+        (void)close(pipedes[0]);
+        (void)close(pipedes[1]);
+        (void)close(devnull); 
+	if (execl(entropy_source[gatherer_entry].path,
 	      entropy_source[gatherer_entry].args[0],
 	      entropy_source[gatherer_entry].args[1],
 	      entropy_source[gatherer_entry].args[2],
 	      entropy_source[gatherer_entry].args[3],
 	      entropy_source[gatherer_entry].args[4],
-	      NULL);
-	_exit(0);
+	      NULL) == -1)
+	{
+		(void)snprintf(msg_buf, sizeof(msg_buf),
+			"Failed to execl(%s, %s, %s, %s, %s, %s)",
+			entropy_source[gatherer_entry].path,
+			entropy_source[gatherer_entry].args[0],
+			entropy_source[gatherer_entry].args[1],
+			entropy_source[gatherer_entry].args[2],
+			entropy_source[gatherer_entry].args[3],
+			entropy_source[gatherer_entry].args[4]);
+		msg_buf[sizeof(msg_buf) - 1] = '\0';
+		error_fatal(msg_buf, errno);
+		_exit(EXIT_FAILURE);
+	}
       }
       else {
 	/*
 	 * Parent
 	 */
-	close(pipedes[1]);
+	(void)close(pipedes[1]);
 	gather_fd = pipedes[0];
 	count_openfd++;
 	non_blocking(gather_fd, 1);
 	gather_start = time(NULL);
 	if (debug)
 	{
-	  fprintf(stderr, "Spawned gatherer: %s (%s",
+	  (void)fprintf(stderr, "Spawned gatherer: %s (%s",
 		entropy_source[gatherer_entry].path,
 		entropy_source[gatherer_entry].args[0]);
 	  i = 1;
-	  while ((i < 5) && (entropy_source[gatherer_entry].args[i]))
-	    fprintf(stderr, " %s", entropy_source[gatherer_entry].args[i++]);
-	  fprintf(stderr, ")\n");
+	  while ((i < 5) && (entropy_source[gatherer_entry].args[i])) {
+	    (void)fprintf(stderr, " %s",
+	    	entropy_source[gatherer_entry].args[i++]);
+	  }
+	  (void)fprintf(stderr, ")\n");
 	}
       }
       continue;
@@ -362,7 +407,8 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
       if (FD_ISSET(egd_query[i].fd, &efds))
       {
 	if (debug)
-	  fprintf(stderr, "Error flagged for service fd %d\n", egd_query[i].fd);
+	  (void)fprintf(stderr, "Error flagged for service fd %d\n",
+	  	egd_query[i].fd);
 	egd_query[i].state = to_be_error_closed;
       }
       else if (FD_ISSET(egd_query[i].fd, &rfds) ||
@@ -403,9 +449,9 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 	{
 	  egd_query[i].buffer[0] = 12;
 #ifdef PID_T_IS_LONG_INT
-	  sprintf((char *) (egd_query[i].buffer + 1), "%12ld", getpid());
+	  (void)sprintf((char *) (egd_query[i].buffer + 1), "%12ld", getpid());
 #else
-	  sprintf((char *) (egd_query[i].buffer + 1), "%12d", getpid());
+	  (void)sprintf((char *) (egd_query[i].buffer + 1), "%12d", (int)getpid());
 #endif
 	  egd_query[i].to_process = 13;
 	  egd_query[i].yet_processed = 0;
@@ -424,7 +470,7 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 	    numbytes = egd_query[i].secbyte;
 	  else
 	    numbytes = 0;
-	  egd_query[i].buffer[0] = numbytes;
+	  egd_query[i].buffer[0] = (char)numbytes;
 	  egd_query[i].to_process = numbytes + 1;
 	  egd_query[i].yet_processed = 0;
 	  egd_query[i].state = wait_write;
@@ -493,7 +539,7 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 	  egd_query[i].yet_processed += error;
 	if (egd_query[i].yet_processed == egd_query[i].to_process) {
 	  egd_query[i].state = wait_command;
-	  fprintf(stderr, "Sent %d bytes to service %d\n",
+	  (void)fprintf(stderr, "Sent %d bytes to service %d\n",
 		egd_query[i].to_process, egd_query[i].fd);
 		}
 	break;
@@ -506,17 +552,22 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
       if ((egd_query[i].state == to_be_closed) ||
 	  (egd_query[i].state == to_be_error_closed)) {
 	if (debug)
-	  fprintf(stderr, "Closing service fd %d, error=%d\n",
+	  (void)fprintf(stderr, "Closing service fd %d, error=%d\n",
 			  egd_query[i].fd,
 			  (egd_query[i].state == to_be_error_closed));
-	else if (egd_query[i].state == to_be_error_closed)
-	  syslog(LOG_INFO, "closing service fd %d for error.", egd_query[i].fd);
-	close(egd_query[i].fd);
+	else if (egd_query[i].state == to_be_error_closed) {
+	  (void)snprintf(msg_buf, sizeof(msg_buf),
+	  	"closing service fd %d for error.", egd_query[i].fd);
+	  msg_buf[sizeof(msg_buf) - 1] = '\0';
+	  error_notice(msg_buf, 0);
+	}
+	(void)close(egd_query[i].fd);
 	count_openfd--;
-	memset(egd_query[i].buffer, 0, 256);	/* Remove delivered entropy */
+	/* Remove delivered entropy */
+	(void)memset(egd_query[i].buffer, 0, 256);
 	if (i < max_query - 1)
 	{
-	  memmove(egd_query + i, egd_query + max_query - 1,
+	  (void)memmove(egd_query + i, egd_query + max_query - 1,
 		  sizeof(egd_query_t));
 	  i--;
 	}
@@ -527,14 +578,11 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
     for (i = 0; i < numsock; i++)
     {
       if (count_openfd >= limit_openfd - OPEN_FD_RESERVE) {
-	if (debug)
-	  fprintf(stderr,
-		  "Could not accept connection: no more slots (%d of %d)\n",
-		  count_openfd, limit_openfd - OPEN_FD_RESERVE);
-	else
-	  syslog(LOG_INFO,
-		  "Could not accept connection: no more slots (%d of %d)",
-		  count_openfd, limit_openfd - OPEN_FD_RESERVE);
+	(void)snprintf(msg_buf, sizeof(msg_buf),
+		"could not accept connection: no more slots (%d of %d)\n",
+		count_openfd, limit_openfd - OPEN_FD_RESERVE);
+	msg_buf[sizeof(msg_buf) - 1] = '\0';
+	error_notice(msg_buf, 0);
 	continue;	/* do not accept new connection */
       }
       if (FD_ISSET(service_socket[i], &rfds))
@@ -565,7 +613,8 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 		        (max_query + 1) * sizeof(egd_query_t));
 	  if (!egd_query)
 	  {
-	    close(accept_fd);	/*
+	    error_error("realloc() failed", errno);
+	    (void)close(accept_fd);	/*
 				 * Better this than complete shut down 
 				 */
 	    egd_query = egd_query_old;
@@ -580,8 +629,10 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
         max_query++;
         egd_query[max_query - 1].fd = accept_fd;
         egd_query[max_query - 1].state = wait_command;
-	if (debug)
-          fprintf(stderr, "Opening service fd %d\n", egd_query[max_query-1].fd);
+	if (debug) {
+          (void)fprintf(stderr, "Opening service fd %d\n",
+	  	egd_query[max_query-1].fd);
+	}
       }
     }
 
